import pandas as pd
from ..pyaver.utils import *

def print_inventory(aver_dexmarket):

    print(f'------------ Inventory -----------')
    print(f'Lamports: {aver_dexmarket.aver_market.client.get_lamports_balance()}')
    print(f'QUOTE: Wallet {aver_dexmarket.aver_market.base_token_balance()}')
    for outcome in aver_dexmarket.dex_outcomes:
        print(f'OUTCOME: {outcome.aver_outcome.name}: Wallet {outcome.get_outcome_wallet_balance()} |  Unsettled: {outcome.get_outcome_open_orders_free_balance()} | Tot: {outcome.get_outcome_open_orders_total_balance()}')
    print(f'----------------------------------')


def get_bids_df(aver_dexoutcome):
    bids = aver_dexoutcome.load_bids()
    bids_df = pd.DataFrame(bids)
    if len(bids_df)>0:
        bids_df[['price','size','price_lots','size_lots']] = bids_df['info'].tolist()
        bids_df['order_object'] = [x for x in bids]
        bids_df['value'] = bids_df['price']*bids_df['size']
        bids_df['cum_vol'] = bids_df['size'].cumsum() 
        bids_df['cost_at_this_price'] = bids_df['size'] * bids_df['price']
        bids_df['cum_total_cost'] = bids_df['cost_at_this_price'].cumsum()
        bids_df['avg_cost'] = bids_df['cum_total_cost']/bids_df['cum_vol']

    return bids_df

def get_asks_df(aver_dexoutcome):
    asks = aver_dexoutcome.load_asks()
    asks_df = pd.DataFrame(asks)
    if len(asks_df)>0:
        asks_df[['price','size','price_lots','size_lots']] = asks_df['info'].tolist()
        asks_df['order_object'] = [x for x in asks]
        asks_df['value'] = asks_df['price']*asks_df['size']
        asks_df['cum_vol'] = asks_df['size'].cumsum() 
        asks_df['cost_at_this_price'] = asks_df['size'] * asks_df['price']
        asks_df['cum_total_cost'] = asks_df['cost_at_this_price'].cumsum()
        asks_df['avg_cost'] = asks_df['cum_total_cost']/asks_df['cum_vol']

    return asks_df
    
def get_mid_price(bids_df, asks_df, default_fair_price):
    best_bid = bids_df['price'].max() if len(bids_df) >0 else None
    best_ask = asks_df['price'].min() if len(asks_df) >0 else None
    if (best_bid == None and best_ask == None):
        mid_price = default_fair_price
    elif best_bid == None:
        mid_price = best_ask
    elif best_ask == None:
        mid_price = best_bid
    else:
        mid_price = (best_bid + best_ask)/2 
    
    return mid_price

def get_own_bids_df(aver_dexoutcome):
    bids = aver_dexoutcome.load_bids()
    bids_df = pd.DataFrame(bids)
    if len(bids_df)>0:
        bids_df[['price','size','price_lots','size_lots']] = bids_df['info'].tolist()
        bids_df['order_object'] = [x for x in bids]
        bids_df['value'] = bids_df['price']*bids_df['size']
        bids_df['cum_vol'] = bids_df['size'].cumsum() 
        bids_df['cost_at_this_price'] = bids_df['size'] * bids_df['price']
        bids_df['cum_total_cost'] = bids_df['cost_at_this_price'].cumsum()
        bids_df['avg_cost'] = bids_df['cum_total_cost']/bids_df['cum_vol']
        own_bids = bids_df[bids_df.open_order_address.astype(str).isin(aver_dexoutcome.open_orders_addresses)]
        available_units_bids = bids_df['size'].sum()
        total_bids_open = own_bids['size'].sum()
        quote_units_open = own_bids['value'].sum()
    else:
        own_bids = pd.DataFrame()
        available_units_bids = 0
        total_bids_open = 0
        quote_units_open = 0

    return own_bids

def get_own_asks_df(aver_dexoutcome):
    asks = aver_dexoutcome.load_asks()
    asks_df = pd.DataFrame(asks)
    if len(asks_df)>0:
        asks_df[['price','size','price_lots','size_lots']] = asks_df['info'].tolist()
        asks_df['order_object'] = [x for x in asks]
        asks_df['value'] = asks_df['price']*asks_df['size']
        asks_df['cum_vol'] = asks_df['size'].cumsum() 
        asks_df['cost_at_this_price'] = asks_df['size'] * asks_df['price']
        asks_df['cum_total_cost'] = asks_df['cost_at_this_price'].cumsum()
        asks_df['avg_cost'] = asks_df['cum_total_cost']/asks_df['cum_vol']
        own_asks = asks_df[asks_df.open_order_address.astype(str).isin(aver_dexoutcome.open_orders_addresses)]
        available_units_asks = asks_df['size'].sum()
        total_asks_open = own_asks['size'].sum()
        base_units_open = total_asks_open
    else:
        own_asks = pd.DataFrame()
        available_units_asks = 0
        total_asks_open = 0
        base_units_open = total_asks_open

    return own_asks

def get_fill_price_bids(bids_df, n):
    tot_n = n
    if len(bids_df)>0:
        for i, r in bids_df.iterrows():
            bids_df.loc[i, 'vol_filled'] = min(r['size'], n) if r.cum_vol < n else min(n, r.cum_vol)
            bids_df.loc[i, 'cost_at_this_price'] = bids_df.loc[i, 'vol_filled'] * r.price
            n = n - bids_df.loc[i, 'vol_filled']
        bids_df['cum_total_cost'] = bids_df['cost_at_this_price'].cumsum()
        tot_filled = bids_df['vol_filled'].sum()
        tot_cost_to_fill = max(bids_df['cum_total_cost'])
        avg_fill_price = safe_div(tot_cost_to_fill, tot_filled)
        return avg_fill_price
    else:
        return None

def get_fill_price_asks(asks_df, n):
    tot_n = n
    if len(asks_df) >0:
        for i, r in asks_df.iterrows():
            asks_df.loc[i, 'vol_filled'] = min(r['size'], n) if r.cum_vol < n else min(n, r.cum_vol)
            asks_df.loc[i, 'cost_at_this_price'] = asks_df.loc[i, 'vol_filled'] * r.price
            n = n - asks_df.loc[i, 'vol_filled']
        asks_df['cum_total_cost'] = asks_df['cost_at_this_price'].cumsum()
        tot_filled = asks_df['vol_filled'].sum()
        tot_cost_to_fill = max(asks_df['cum_total_cost'])
        avg_fill_price = safe_div(tot_cost_to_fill, tot_filled)
        return avg_fill_price
    else:
        return None

def get_price_to_fill_bids(bids_df, n):
    tot_n = n
    if len(bids_df)>0:
        for i, r in bids_df.iterrows():
            bids_df.loc[i, 'vol_filled'] = min(r['size'], n) if r.cum_vol < n else min(n, r.cum_vol)
            bids_df.loc[i, 'cost_at_this_price'] = bids_df.loc[i, 'vol_filled'] * r.price
            n = n - bids_df.loc[i, 'vol_filled']
            if n <= 0:
                return r.price
        return None
    else:
        return None

def get_price_to_fill_asks(asks_df, n):
    tot_n = n
    if len(asks_df) >0:
        for i, r in asks_df.iterrows():
            asks_df.loc[i, 'vol_filled'] = min(r['size'], n) if r.cum_vol < n else min(n, r.cum_vol)
            asks_df.loc[i, 'cost_at_this_price'] = asks_df.loc[i, 'vol_filled'] * r.price
            n = n - asks_df.loc[i, 'vol_filled']
            if n <= 0:
                return r.price
        return None
    else:
        return None