
from pyserum.market import Market as SerumMarket
from pyserum.enums import OrderType, Side
from solana.account import Account
from solana.publickey import PublicKey
from solana.rpc.api import Client
import base64
import base58
import time
import random
import requests
  
from ..pyaver.core import AverClient, AverMarket, AverOutcome
from ..pyaver.averdex import AverDeXMarket, AverDeXOutcome
from ..pyaver.utils import *
from .bot_utils import *


def punter(account_secretkey, parameters):

    print('****************** PUNTER REFRESH *********************')

    trade = True
    
    solana_url = 'https://api.devnet.solana.com'
    owner = Account(base58.b58decode(account_secretkey))

    aver_url = parameters['aver_url']
    max_size = parameters['max_size']                 
    min_balance_currency = parameters['min_balance_currency']
    min_balance_lamports = parameters['min_balance_lamports']
    market_id = int(parameters['aver_market_id'])
    
    order_type = OrderType.IOC 

    aver_client = AverClient(
        solana_client=solana_url,
        solana_account=owner
    )
    aver_market = AverMarket(
        aver_client=aver_client,
        aver_market_id=market_id,
    )
    aver_dexmarket = AverDeXMarket(
        aver_market=aver_market,
        auto_mint=True,
    )

    print(f"Market: {aver_market.name}")

    fair_price = float(1/aver_market.number_outcomes) # to be derived somehow... (1/number_outcomes, or external data?)

    
    # -------------------------------------------------------
    #Checking minimum balance criteria are met...

    topup_required = False
    #print_inventory(aver_dexmarket)

    lamport_balance = aver_client.get_lamports_balance()
    if lamport_balance < min_balance_lamports:
        topup_required = True
        print(f"Lamports < min ({lamport_balance} < {min_balance_lamports}")
        print(f" -> Airdropping {min_balance_lamports} lamports ")
        aver_client.request_lamports_airdrop(min_balance_lamports)

    market_base_token_balance = aver_market.base_token_balance() 
    if market_base_token_balance < min_balance_currency:
        topup_required = True
        print(f"Quote wallet balance < min ({market_base_token_balance} < {min_balance_currency}")
        print(f" -> Airdropping {min_balance_currency} {aver_market.base_token_name}")
        aver_market.request_base_token_airdrop(min_balance_currency)

    if topup_required:
        print_inventory(aver_dexmarket)
        pass

    aver_market.create_atas_if_required()

    for idx, aver_dexoutcome in enumerate(aver_dexmarket.dex_outcomes):
        # -------------------------------------------------------
        # Get market info, and decide what to do...

        asks_df = get_asks_df(aver_dexoutcome)
        bids_df = get_bids_df(aver_dexoutcome)
        mid_price = get_mid_price(bids_df, asks_df, fair_price)

        # Decide to buy or sell based on probability weighted random draw (should regress to 'fair')
        desired_trade_size = round_nearest_step(random.random()*max_size, aver_dexoutcome.price_tick_size)
        standardised_price = ((mid_price - fair_price)/0.16667)
        rand = 2*(random.random()-0.5)

        print(f"Desired units {desired_trade_size} | Mid: {mid_price} | {standardised_price} v {rand}")
        if rand > standardised_price:
            # Cap trade size based on available asks, and quote tokens available to pay
            side = Side.BUY
            ask_capacity = asks_df['size'].sum()
            capped_trade_size = min(ask_capacity, desired_trade_size)
            if capped_trade_size > 0:
                trade = True
                ask_avg_fill_price = get_fill_price_asks(asks_df, capped_trade_size)
                quote_tokens_required = capped_trade_size * ask_avg_fill_price
                quote_tokens_held = aver_dexmarket.aver_market.base_token_balance()
                order_size = round_nearest_step(min(0.9*safe_div(quote_tokens_held, ask_avg_fill_price), capped_trade_size), aver_dexoutcome.increment_size)
                price = get_price_to_fill_asks(asks_df, order_size)
        else:
            # Cap trade size based on available bids, and base tokens available to sell
            side = Side.SELL
            bid_capacity = bids_df['size'].sum()
            capped_trade_size = min(bid_capacity, desired_trade_size)
            if capped_trade_size >0:
                trade = True
                bid_avg_fill_price = get_fill_price_bids(bids_df, capped_trade_size)
                base_tokens_required = capped_trade_size
                base_tokens_held = aver_dexoutcome.get_outcome_wallet_balance()
                order_size = round_nearest_step(min(0.9*base_tokens_held, capped_trade_size), aver_dexoutcome.increment_size)
                price = get_price_to_fill_bids(bids_df, order_size)
        

        if trade and order_size > 0:
            print(f"Order : {side.name} {order_size} @ {price}")
            aver_dexoutcome.place_order(side, price, order_size, order_type, client_id=0)
        else:
            print(f"No trade - insufficient balances to support desired trade direction.")

        # Crank market
        aver_dexoutcome.crank_market()
        # Wait for settle...
        time.sleep(5)

        # Settle if anything to settle
        if aver_dexoutcome.get_outcome_open_orders_free_balance() > 0 or aver_dexoutcome.get_quote_open_orders_free_balance() > 0:
            print('Settling...')
            aver_dexoutcome.settle_balances()

    print('****************** COMPLETED REFRESH *********************')



