from pyaver.core import AverClient, AverMarket, AverOutcome
from pyaver.averdex import AverDeXMarket, AverDeXOutcome
from solana.account import Account 
import base58

from pyaver.utils import await_confirmation
import requests

# Creates new user account
user_account = Account()
print(f"User account: {user_account.public_key()}")

# Create an AverClient on devnet
client = AverClient(
        solana_client='https://api.devnet.solana.com',
        solana_account=user_account
    )

# Request an airdrop of 1 SOL
client.request_lamports_airdrop(1)
print(client.get_lamports_balance())

# Get all actively traded markets, then get first market
AVER_BASE_URL = 'https://aver.bet'
markets = client.list_active_trading_markets()
print(markets)
first_market_id = markets[0]['id']

# Create an AverMarket instance for the first active trading market
market = AverMarket(aver_client=client,  aver_market_id=first_market_id)
print(market.name)
print(market.data)
print(market.outcomes)

# Request an airdrop of 1000 base tokens for this market
market.request_base_token_airdrop(1000)
print(market.base_token_balance())

# Mint 100 outcome tokensets for this market
market.mint(100)

for o in market.outcomes:
    print(f"Balance of {o.name}: {o.balance()} after mint")

# Redeem some of these tokensets
market.redeem(10)

for o in market.outcomes:
    print(f"Balance of {o.name}: {o.balance()} after redeem")

# Create an AverDeX instance
averdexmarket = AverDeXMarket(market, auto_mint=True, auto_settle=False)
print(averdexmarket)
print(averdexmarket.aver_market.name)

# Trade some tokens
qty = 5
price = 1/market.number_outcomes

for o in averdexmarket.dex_outcomes:
    o.place_order(
        side= Side.BUY,
        price=price,
        quantity=qty,
        order_type=OrderType.LIMIT,
    )
    qty+=1
    print(f"Placing buy order for {o.name} x {qty} @ {price}")

    # Get open orders
    for oo in o.get_open_orders():
        print(oo)

    # Crank the market, as on devnet
    o.crank_market()

    # Settle balances, if any
    o.settle_balances()
    